<?php

if (!defined('_PS_VERSION_'))
	exit;

class ffcheckbox extends Module
{
	protected $_html = '';
	protected $_postErrors = array();
    public $api_url;
    public $test_api_url = 'https://dev-api.checkbox.in.ua';
    public $prod_api_url = 'https://api.checkbox.in.ua';

	public function __construct()
	{
		$this->name = 'ffcheckbox';
        $this->tab = 'advertising_marketing';
		$this->version = '1.1.0';
		$this->author = 'Forforce';

		$this->bootstrap = true;
		parent::__construct();

		$this->displayName = $this->l('Checkbox');
		$this->description = $this->l('Accept payments for your products via Checkbox service.');
		$this->confirmUninstall = $this->l('Are you sure about removing these details?');
		$this->ps_versions_compliancy = array('min' => '1.6', 'max' => '1.6.99.99');

		if(ConfigurationCore::get('PS_FFCHECKBOX_SANDBOX') != 1){
            $this->api_url = $this->prod_api_url;
        }else{
            $this->api_url = $this->test_api_url;
        }

	}

	public function install()
	{
		if (!parent::install() || !$this->installSql() || !$this->registerHook('displayAdminOrder') || !$this->registerHook('actionOrderStatusPostUpdate'))
			return false;

        Configuration::updateValue('PS_FFCHECKBOX_SANDBOX', 1);
        Configuration::updateValue('PS_FFCHECKBOX_AUTH_TOKEN', '');
//        Configuration::updateValue('PS_FFCHECKBOX_SHIFT', '');
        return true;
	}
	public function installSql(){
        return Db::getInstance()->execute('
				CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'ffcheckboxpayment` (
				`id_ffcheckboxpayment` INT UNSIGNED NOT NULL AUTO_INCREMENT,
				`id_order` int(10) unsigned NOT NULL,
				`id_receipt` VARCHAR(191)  NOT NULL,
				PRIMARY KEY (`id_ffcheckboxpayment`)
			) ENGINE='._MYSQL_ENGINE_.' DEFAULT CHARSET=utf8 ;'
        );
    }

	public function uninstall()
	{
		if (!parent::uninstall())
			return false;
		return true;
	}

    function hookActionOrderStatusPostUpdate($params)
    {
        $create_receipt = Configuration::get('PS_FFCHECKBOX_SEND_ORDER_UPDATE');
        $useStatus = explode(',', Configuration::get('PS_FFCHECKBOX_SEND_ORDER_STATES'));
        if ($create_receipt == 1 && in_array((int)$params['newOrderStatus']->id, $useStatus)
            && empty(DB::getInstance()->getValue('SELECT id_receipt FROM '._DB_PREFIX_.'ffcheckboxpayment WHERE id_order = '.(int)$params['id_order']))
        ) {
            $this->createReceipt((int)$params['id_order']);
        }
    }


	protected function _postValidation()
	{
		if (Tools::isSubmit('btnSubmit'))
		{
			if (!Tools::getValue('PS_FFCHECKBOX_LOGIN'))
				$this->_postErrors[] = $this->l('Account login are required.');
			elseif (!Tools::getValue('PS_FFCHECKBOX_PASSWORD'))
				$this->_postErrors[] = $this->l('Account password is required.');
			elseif (!Tools::getValue('PS_FFCHECKBOX_LICENCE_KEY'))
				$this->_postErrors[] = $this->l('Account licence key is required.');
		}
	}

	protected function _postProcess()
	{
		if (Tools::isSubmit('btnSubmit'))
		{
			Configuration::updateValue('PS_FFCHECKBOX_LOGIN', Tools::getValue('PS_FFCHECKBOX_LOGIN'));
			Configuration::updateValue('PS_FFCHECKBOX_PASSWORD', Tools::getValue('PS_FFCHECKBOX_PASSWORD'));
			Configuration::updateValue('PS_FFCHECKBOX_LICENCE_KEY', Tools::getValue('PS_FFCHECKBOX_LICENCE_KEY'));
			Configuration::updateValue('PS_FFCHECKBOX_SANDBOX', Tools::getValue('PS_FFCHECKBOX_SANDBOX'));
			Configuration::updateValue('PS_FFCHECKBOX_SEND_ORDER_UPDATE', Tools::getValue('PS_FFCHECKBOX_SEND_ORDER_UPDATE'));
			Configuration::updateValue('PS_FFCHECKBOX_SEND_ORDER_STATES', implode(',',Tools::getValue('id_order_state')));

			if(!$this->apiAuth()){
                $this->_postErrors[] = $this->l('Wrong login and password.');
            }
		}
		$this->_html .= $this->displayConfirmation($this->l('Settings updated'));
	}



	public function renderForm()
	{
        $orderStates = OrderStateCore::getOrderStates(Context::getContext()->language->id);
		$fields_form = array(
			'form' => array(
				'legend' => array(
					'title' => $this->l('Checkbox service details'),
					'icon' => 'icon-envelope'
				),
				'input' => array(
					array(
						'type' => 'text',
						'label' => $this->l('Account login'),
						'name' => 'PS_FFCHECKBOX_LOGIN',
						'required' => true
					),
                    array(
						'type' => 'text',
						'label' => $this->l('Account password'),
						'name' => 'PS_FFCHECKBOX_PASSWORD',
						'required' => true
					),
                    array(
						'type' => 'text',
						'label' => $this->l('Licence key'),
						'name' => 'PS_FFCHECKBOX_LICENCE_KEY',
						'required' => true
					),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Sandbox'),
                        'name' => 'PS_FFCHECKBOX_SANDBOX',
                        'required' => true,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'PS_FFCHECKBOX_SANDBOX',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'PS_FFCHECKBOX_SANDBOX',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        )
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Create receipt on status change'),
                        'name' => 'PS_FFCHECKBOX_SEND_ORDER_UPDATE',
                        'required' => true,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'PS_FFCHECKBOX_SEND_ORDER_UPDATE',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'PS_FFCHECKBOX_SEND_ORDER_UPDATE',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        )
                    ),
                    array(
                        'type' => 'select' ,
                        'label' => $this->l('Order states:') ,
                        'name' => 'id_order_state[]',
                        'required' => false ,
                        'multiple' => true ,
                        'class' => 'chosen' ,
                        'options' => array(
                            'query' => $orderStates ,
                            'id' => 'id_order_state' ,
                            'name' => 'name' ,
                        )
                    )
				),
				'submit' => array(
					'title' => $this->l('Save'),
				),
                'buttons' => array(
                    'cancelShift' => array(
                        'title' => $this->l('Закрыть смену'),
                        'name' => 'cancelShift',
                        'type' => 'submit',
                        'class' => 'btn btn-default pull-right',
                        'icon' => 'process-icon-cancel',
                    ),
                ),
			),
		);

		$helper = new HelperFormCore();
		$helper->show_toolbar = false;
		$helper->table = $this->table;
		$lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
		$helper->default_form_language = $lang->id;
		$helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
		$this->fields_form = array();
		$helper->id = (int)Tools::getValue('id_carrier');
		$helper->identifier = $this->identifier;
		$helper->submit_action = 'btnSubmit';
		$helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
		$helper->token = Tools::getAdminTokenLite('AdminModules');
		$helper->tpl_vars = array(
			'fields_value' => $this->getConfigFieldsValues(),
			'languages' => $this->context->controller->getLanguages(),
			'id_language' => $this->context->language->id
		);

		return $helper->generateForm(array($fields_form));
	}

	public function getConfigFieldsValues()
	{
		return array(
			'PS_FFCHECKBOX_LOGIN' => Tools::getValue('PS_FFCHECKBOX_LOGIN', Configuration::get('PS_FFCHECKBOX_LOGIN')),
			'PS_FFCHECKBOX_PASSWORD' => Tools::getValue('PS_FFCHECKBOX_PASSWORD', Configuration::get('PS_FFCHECKBOX_PASSWORD')),
			'PS_FFCHECKBOX_LICENCE_KEY' => Tools::getValue('PS_FFCHECKBOX_LICENCE_KEY', Configuration::get('PS_FFCHECKBOX_LICENCE_KEY')),
			'PS_FFCHECKBOX_SANDBOX' => Tools::getValue('PS_FFCHECKBOX_SANDBOX', Configuration::get('PS_FFCHECKBOX_SANDBOX')),
			'PS_FFCHECKBOX_SEND_ORDER_UPDATE' => Tools::getValue('PS_FFCHECKBOX_SEND_ORDER_UPDATE', Configuration::get('PS_FFCHECKBOX_SEND_ORDER_UPDATE')),
            'id_order_state[]' => explode(',',Configuration::get('PS_FFCHECKBOX_SEND_ORDER_STATES'))
		);
	}

    public function hookdisplayAdminOrder($params)
    {
        $order = new Order($params['id_order']);

        $shift = $this->checkShift();
        $context = Context::getContext();
        $id_check = DB::getInstance()->getValue('SELECT id_receipt FROM '._DB_PREFIX_.'ffcheckboxpayment WHERE id_order = '.$params['id_order']);
        $token = Tools::getAdminTokenLite('AdminOrders');
        Configuration::updateValue('FFCHECKBOX_TOKEN_'.Context::getContext()->employee->id, $token);
        $this->smarty->assign(array(
            'module_dir' => _MODULE_DIR_ . $this->name,
            'id_order' => $order->id,
            'token' => $token,
            'id_employee' => Context::getContext()->employee->id,
            'baseurl' => $this->context->shop->getBaseURL(true, true),
            'shift' => $shift,
            'id_check' => $id_check,
        ));
        return $this->display(__FILE__, 'admin_order.tpl');
    }

    public function apiSendRequest($url, $json_value = null, $headers = array(), $token = false, $method = 'POST', $return_json = true) {
        $ch = curl_init();

        if($method == 'POST') {
            curl_setopt($ch, CURLOPT_POST, 1);
        }

        if ($json_value) {
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($json_value));
            $headers[] = 'Accept: application/json';
            $headers[] = 'Content-Type: application/json';
        }

        curl_setopt($ch, CURLOPT_HEADER, 1);


        if($token)
            $headers[] = 'Authorization: Bearer '.$token;

        $headers[] = 'X-Client-Name: '.$this->author;
        $headers[] = 'X-Client-Version: v'.$this->version;

        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt ($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch,CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_VERBOSE, 1);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        if ($output = curl_exec($ch))
        {
//            d($output);

            $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
            $header = substr($output, 0, $header_size);
            $body = substr($output, $header_size);
            $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);

//            $return = json_decode($body,true);
//            if (preg_match('/HTTP\/1.1 200 OK\b/', $header)
//                || preg_match('/HTTP\/1.1 202\b/', $header)
//                || preg_match('/HTTP\/1.1 201\b/', $header)
//            ) {
            if ($httpcode == 200 || $httpcode == 202 || $httpcode == 201) {
                if($return_json){
                    return  ['status' => 'ok' , 'body' => json_decode($body,true)];
                }

                return  ['status' => 'ok' , 'body' => $body];
            } else
            {
                $this->log($httpcode, $body, $url, true);
                return ['status' => 'error' , 'body' =>  $body];
            }
        } else {
            $this->log(500, 'Превышено время ожидание от сервиса', $url, true);
            return ['status' => 'error' , 'body' => 'Превышено время ожидание от сервиса'];
        }

    }

    public function getContent()
    {
        if (Tools::isSubmit('btnSubmit'))
        {
            $this->_postValidation();
            if (!count($this->_postErrors))
                $this->_postProcess();
            else
                foreach ($this->_postErrors as $err)
                    $this->_html .= $this->displayError($err);

            foreach ($this->_postErrors as $err)
                $this->_html .= $this->displayError($err);
        }

        if(Tools::isSubmit('cancelShift')){
            if($this->closeShift()){
                $this->_html .= $this->displayConfirmation('Смена была успешно закрыта.');
            }else{
                $this->_html .= $this->displayError('Произошла ошибка при закрытии смены.');
            }
        }

        else
            $this->_html .= '<br />';
        $this->_html .= $this->renderForm();
        return $this->_html;
    }

    public function apiAuth(){
	    $login = ConfigurationCore::get('PS_FFCHECKBOX_LOGIN');
        $pass = ConfigurationCore::get('PS_FFCHECKBOX_PASSWORD');
        $url = $this->api_url.'/api/v1/cashier/signin';
        $resp = $this->apiSendRequest($url, ['login'=>$login,'password'=>$pass]);
        if($resp['status'] == 'ok' && isset($resp['body']['access_token'])){
            Configuration::updateValue('PS_FFCHECKBOX_AUTH_TOKEN', $resp['body']['access_token']);
            return $resp['body']['access_token'];
        }else{
//            $this->log(200, $resp['body'], $url);
            Configuration::updateValue('PS_FFCHECKBOX_AUTH_TOKEN', '');
            return false;
        }
    }

    public function log($status, $data, $url, $json = false){
	    if(!$json)
	        $data = json_encode($data);
	    $file = _PS_MODULE_DIR_.'ffcheckbox/log/request.log';
        file_put_contents($file, 'status: '.$status.', url: '.$url.', data: '.$data . "\r\n---------------------\r\n", FILE_APPEND);
    }

    public function getAuthToken(){
        $token = ConfigurationCore::get('PS_FFCHECKBOX_AUTH_TOKEN');
        $date_upd = DB::getInstance()->getValue('SELECT date_upd FROM '._DB_PREFIX_.'configuration WHERE `name` = "PS_FFCHECKBOX_AUTH_TOKEN"');
        $date_upd = \DateTime::createFromFormat('Y-m-d H:i:s', (string)$date_upd);
        $date_now = new \DateTime('now');
        $date_now->modify('-1 days');
        if(empty($token) || !$date_upd || $date_now > $date_upd){
            return $this->apiAuth();
        }
        return $token;
    }

    public function openShift(){
	    $key =  Configuration::get('PS_FFCHECKBOX_LICENCE_KEY');
	    if(!empty($key)) {
            $headers = [
                'X-License-Key: ' . $key
            ];
            $url = $this->api_url.'/api/v1/shifts';
            $token = $this->getAuthToken();
            $resp = $this->apiSendRequest($url, array(), $headers, $token);
            if($resp['status'] == 'ok' && isset($resp['body']['id']) && isset($resp['body']['status']) && $resp['body']['status'] == 'CREATED'){
                Configuration::updateValue('PS_FFCHECKBOX_SHIFT', $resp['body']['id']);
                return $resp['body']['id'];
            }else{
//                $this->log(200, $resp['body'], $url);
                return false;
            }
        }
	    return false;
    }

    public function closeShift(){
        $key =  Configuration::get('PS_FFCHECKBOX_LICENCE_KEY');
        if(!empty($key)) {
            $headers = [
                'X-License-Key: ' . $key
            ];
            $url = $this->api_url.'/api/v1/shifts/close';
            $token = $this->getAuthToken();
            $resp = $this->apiSendRequest($url, array(), $headers, $token);
            if($resp['status'] == 'ok' && isset($resp['body']['id']) && isset($resp['body']['status']) && $resp['body']['status'] == 'CLOSING'){
                Configuration::updateValue('PS_FFCHECKBOX_SHIFT', '');
                return $resp['body']['id'];
            }else{
//                $this->log(200, $resp['body'],$url);
                return false;
            }
        }
        return false;
    }

    public function checkShift(){
        $shift =  Configuration::get('PS_FFCHECKBOX_SHIFT');
        if(!empty($shift)) {
            $url = $this->api_url.'/api/v1/shifts/'.$shift;
            $token = $this->getAuthToken();
            $headers = array();
            $headers[] = 'Accept: application/json';
            $headers[] = 'Content-Type: application/json';

            $resp = $this->apiSendRequest($url, array(), $headers, $token, 'GET');
            if($resp['status'] == 'ok' && isset($resp['body']['id']) && isset($resp['body']['status']) && $resp['body']['status'] == 'OPENED'){
                return true;
            }elseif($resp['status'] == 'ok' && isset($resp['body']['id']) && isset($resp['body']['status']) && $resp['body']['status'] == 'CREATED'){
                return 'waiting';
            }else{
                Configuration::updateValue('PS_FFCHECKBOX_SHIFT', '');
            }
        }else{
            $shifts = $this->getShifts();
            foreach ($shifts as $shift){
                if($shift['status'] == 'OPENED'){
                     Configuration::updateValue('PS_FFCHECKBOX_SHIFT', $shift['id']);
                     return true;
                }
            }
        }


        if($this->openShift()){
            return 'waiting';
        }else{
            $shifts = $this->getShifts();
            foreach ($shifts as $shift){
                if($shift['status'] == 'OPENED'){
                    Configuration::updateValue('PS_FFCHECKBOX_SHIFT', $shift['id']);
                    return true;
                }
            }
        }
        return false;
    }

    public function createReceipt($id_order){
	    $order = new Order($id_order);
        $customer = new Customer($order->id_customer);
	    $details = $order->getOrderDetailList();
        $cashier_data = $this->getCashierData();
        $products_array = array();
        $discounts = array();
        $payments = array();
        $total_sum = 0;

        foreach ($details as $detail){
            $total_sum += (float)($detail['total_price_tax_incl']/(int)$detail['product_quantity']) * (int)$detail['product_quantity'];

            $products_array[] = [
                'good' => [
                    'code' => $detail['product_reference'],
                    'name' => $detail['product_name'],
                    'price' => (float)($detail['total_price_tax_incl']/(int)$detail['product_quantity']) * 100
                ],
//                'good_id' => $detail['product_id'],
                'quantity' => (int)$detail['product_quantity']*1000
            ];
        }
        $cart_rules = $order->getCartRules();
        foreach ($cart_rules as $cart_rule){
            $discounts[] = [
                'type' => 'DISCOUNT',
                'mode' => 'VALUE',
                'value' => (float)$cart_rule['value']*100,
                'name' => $cart_rule['name'],
            ];

            $total_sum -= (float)$cart_rule['value'];
        }
        $payments[] = [
            'type' => 'CARD',
            'value' => $total_sum*100,
        ] ;
        $data = [
            'cashier_name' => $cashier_data['full_name'],
            'goods' => $products_array,
            'delivery' => ['email' => $customer->email],
            'payments' => $payments,
            'discounts' => $discounts,
        ];
        if($this->checkShift() === true){
            $url = $this->api_url.'/api/v1/receipts/sell';
            $token = $this->getAuthToken();
            $resp = $this->apiSendRequest($url, $data, array(), $token);
            if($resp['status'] == 'ok' && isset($resp['body']['id'])){
                return DB::getInstance()->insert('ffcheckboxpayment',
                    [
                        'id_order' => $id_order,
                        'id_receipt' =>  $resp['body']['id']
                    ]
                );
            }
        }
        return false;
    }

    public function getCashierData(){
        $token =  $this->getAuthToken();
        if(!empty($token)) {
            $url = $this->api_url.'/api/v1/cashier/me';
            $resp = $this->apiSendRequest($url, array(), array(), $token, 'GET');
            if($resp['status'] == 'ok' && isset($resp['body']['id'])){
                return $resp['body'];
            }
        }
        return false;
    }

    public function createLabel($id_order){
	    $id_check = DB::getInstance()->getValue('SELECT id_receipt FROM '._DB_PREFIX_.'ffcheckboxpayment WHERE id_order = '.$id_order);
	    if(!$id_check)
	        return false;
        if($this->checkShift() === true){
            $url = $this->api_url.'/api/v1/receipts/'.$id_check.'/pdf';
            $token = $this->getAuthToken();
            $headers = ['Content-Type: application/pdf'];
            $resp = $this->apiSendRequest($url, null, $headers, $token, 'GET', false);
            if($resp['status'] == 'ok' && !empty($resp['body'])){
                file_put_contents('files/'.$id_check.'.pdf',$resp['body']);
                return $id_check;
            }
        }
        return false;
    }

    public function getShifts(){
        $token =  $this->getAuthToken();
        if(!empty($token)) {
            $url = $this->api_url.'/api/v1/shifts';
            $resp = $this->apiSendRequest($url, array(), array(), $token, 'GET');
            if($resp['status'] == 'ok' && isset($resp['body']['results'])){
                return $resp['body']['results'];
            }
        }
        return false;
    }
}
